library(data.table)
library(dplyr)

# set working directory to the folder with all of the data
setwd("~/PerformancePersistenceCode")

# import mutual fund data
Data<-read.table("mutualfunds.csv", header=TRUE, sep=",", stringsAsFactors = FALSE)
# find year and month for each observation
Data$year<-as.numeric(substr(Data$caldt,1,4))
Data$month<-as.numeric(substr(Data$caldt,5,6))

# want to restrict to equity domestic, excluding sector and hedged and short funds, and excluding pure index funds and ETFs and ETNs
# keepfunds.csv is all of the funds that fit this criteria (exported from keepfunds.R file)
keepfunds<-read.table("keepfunds.csv", header=TRUE, sep=",", stringsAsFactors = FALSE)
keepfunds<-keepfunds[!duplicated(paste0(keepfunds$crsp_fundno, keepfunds$year)),]
Data<-left_join(keepfunds, Data)
Data<-Data[!is.na(Data$caldt),]
rm(keepfunds)

# work with mtna of each fund
# set those with non-numeric tna to 0
Data$mtna<-as.numeric(Data$mtna)
Data$mtna[which(Data$mtna==-99)]<-NA
Data$mtna[which(is.na(Data$mtna)==TRUE)]<-0

# mtna data is annual through 1969, quarterly through 1990, then monthly after that
# we fill in mtna data for observations that don't have it with the most recent annual/quarterly data that we have at that time
Data$quarter<-NA
Data$quarter[which(Data$month %in% c(1,2,3))]<-1
Data$quarter[which(Data$month %in% c(4,5,6))]<-2
Data$quarter[which(Data$month %in% c(7,8,9))]<-3
Data$quarter[which(Data$month %in% c(10,11,12))]<-4
# find most recent annual and quarterly information we have at that time
annual.mtna<-Data[which(Data$month==12), colnames(Data) %in% c("crsp_fundno","year","mtna")]
annual.mtna$year<-annual.mtna$year+1  # increase year by 1 because we use the data from december of the previous year in the next year
colnames(annual.mtna)[3]<-"annualmtna"
quarterly.mtna<-Data[which(Data$month %in% c(3,6,9,12)), colnames(Data) %in% c("crsp_fundno","year","quarter","mtna")]
quarterly.mtna$quarter<-quarterly.mtna$quarter+1 # increase quarter by 1 because we use data from the previous quarter in the next quarter
quarterly.mtna$year[which(quarterly.mtna$quarter==5)]<-quarterly.mtna$year[which(quarterly.mtna$quarter==5)]+1
quarterly.mtna$quarter[which(quarterly.mtna$quarter==5)]<-1
colnames(quarterly.mtna)[3]<-"quarterlymtna"
Data<-left_join(Data, annual.mtna)
Data<-left_join(Data, quarterly.mtna)
Data$quarterlymtna[is.na(Data$quarterlymtna)]<-0
Data$annualmtna[is.na(Data$annualmtna)]<-0
# fill in mtna of those that are missing it with the most recent quarterly (and if still missing, annual) data that we have
Data$mtna[which(Data$mtna==0 & Data$quarterlymtna>0)]<-Data$quarterlymtna[which(Data$mtna==0 & Data$quarterlymtna>0)]
Data$mtna[which(Data$mtna==0 & Data$annualmtna>0)]<-Data$annualmtna[which(Data$mtna==0 & Data$annualmtna>0)]
Data<-Data[,!(colnames(Data) %in% c("annualmtna","quarterlymtna","quarter"))]
rm(annual.mtna, quarterly.mtna)

# work with fund returns
# replace R with 0
Data$mret[Data$mret=="R"]<-0
# calculate 1+r for each month
Data$return<-as.numeric(Data$mret)+1
# remove observations with NA returns
Data<-Data[!is.na(Data$return),]

# crsp reports fund observations by share class, but we want to aggregate it up to the fund level
# first, we aggregate up by wficn if available
# then, we aggregate by crsp class group if available
# crsp class group is available only starting in 1998, but we use earliest available value for pre-1998 years
# entries without a wficn number or crsp class group are treated as individual funds
# first, link crsp_fundno with wficn number
wficn<-read.csv("crspwficn.csv", header=TRUE, sep=",", stringsAsFactors = FALSE)
Data<-left_join(Data, wficn)
# link crsp_fundno with crsp class group
shareclass<-read.table("shareclass.csv", header=TRUE, sep=",", stringsAsFactors = FALSE)
shareclass<-shareclass[!is.na(shareclass$crsp_cl_grp),]
shareclass$year<-as.numeric(substr(shareclass$caldt,1,4))
shareclass<-shareclass[,colnames(shareclass) %in% c("crsp_fundno","crsp_cl_grp","year")]
shareclass<-shareclass[!duplicated(paste0(shareclass$crsp_fundno, shareclass$year)),]
# # make a "last year share class", so that we can aggreage funds in the year they disappear from the sample
shareclass<-shareclass[order(shareclass$crsp_fundno, shareclass$year),]
shareclasslast<-shareclass[!duplicated(shareclass$crsp_fundno, fromLast=T),]
shareclasslast$year<-shareclasslast$year+1
shareclass<-rbind(shareclass, shareclasslast)
shareclass<-shareclass[order(shareclass$crsp_fundno, shareclass$year),]
# # join shareclass with Data
Data<-left_join(Data, shareclass)
# for pre-1998, we assume the fund composition stayed the same between then and the first year it appears in shareclass.
# we take the earlist available value for crsp_cl_grp for all pre-1998 observations
shareclass98<-shareclass[!duplicated(shareclass$crsp_fundno),]
shareclass98<-shareclass98[,colnames(shareclass98) %in% c("crsp_fundno","crsp_cl_grp")]
colnames(shareclass98)[2]<-"crsp_cl_grp_98"
Data<-left_join(Data, shareclass98)
Data$crsp_cl_grp[is.na(Data$crsp_cl_grp)]<-Data$crsp_cl_grp_98[is.na(Data$crsp_cl_grp)]
# if wficn available in a given year, refer to the fund by wficn number
# if wficn not available, refer to fund by crsp shareclass
# if neither available, refer to it by crsp_fundno (treat as individual funds)
Data$crsp_fundno[!is.na(Data$crsp_cl_grp)]<-Data$crsp_cl_grp[!is.na(Data$crsp_cl_grp)]
Data$crsp_fundno[!is.na(Data$wficn)]<-Data$wficn[!is.na(Data$wficn)]
rm(shareclass, shareclasslast, shareclass98, wficn)

# value-weight up all share classes within the same fund
Data$totaltna<-ave(Data$mtna, Data$year, Data$month, Data$crsp_fundno, FUN=sum)
Data$sharetna<-Data$mtna/Data$totaltna
Data$shareret<-Data$sharetna*Data$return
Data$totalreturn<-ave(Data$shareret, Data$year, Data$month, Data$crsp_fundno, FUN=sum)
# for those with 0 totaltna, set return to be the equal-weighted average of the share classes
tna0<-Data[which(Data$totaltna==0),]
tna0$totalreturn<-ave(tna0$return, tna0$year, tna0$month, tna0$crsp_fundno, FUN=mean)
Data<-Data[which(Data$totaltna>0),]
Data<-rbind(Data, tna0)
rm(tna0)
# keep only one entry for each fund, and the relevant columns
Data$mtna<-Data$totaltna
Data$return<-Data$totalreturn
Data<-Data[!duplicated(paste0(Data$crsp_fundno, Data$year, Data$month)), c("crsp_fundno","year","month","return","mtna")]

# save Data
Data$ym<-paste0(Data$year, Data$month)
Data<-Data[,c("crsp_fundno","ym","return","mtna")]
colnames(Data)[1]<-"fund_id"
write.csv(Data, row.names=FALSE, file="MFmonthlyreturns.csv")
