library(data.table)
library(dplyr)

setwd("~/PerformancePersistenceCode")

Data<-read.csv("MFmonthlyreturns.csv", header=TRUE, sep=",", stringsAsFactors = FALSE)
Data$year<-as.numeric(substr(Data$ym,1,4))
Data$month<-as.numeric(substr(Data$ym,5,nchar(Data$ym)))
Data<-Data[, c("fund_id","year","month","return","mtna")]

# calculate year-to-date return for each fund
Data<-Data[order(Data$fund_id, Data$year, Data$month),]
Data$ytdreturn<-with(Data, ave(Data$return, Data$fund_id, Data$year, FUN=cumprod))
# when looking at annual returns, keep on the final value in each year, and only keep if the fund was in the sample for the whole 12 months
fylength<-rle(paste0(Data$fund_id, Data$year))$lengths
fypair<-rle(paste0(Data$fund_id, Data$year))$values
keep.values<-cumsum(fylength)
annual.return<-Data[keep.values, c("fund_id","year","ytdreturn","mtna")]
annual.return$nummonth<-fylength
# remove all without full 12 months
annual.return<-annual.return[which(annual.return$nummonth==12),]
# keep only funds with >0 tna
annual.return<-annual.return[which(annual.return$mtna>0),]

# sort by year and then by return in order to place into bins by previous year's return
annual.return<-annual.return[order(annual.return$year, annual.return$ytdreturn),]
# find number of funds for each year
funds.per.year<-rle(annual.return$year)$length
years<-rle(annual.return$year)$values
# split into 10 buckets (with 1A,1B,1C,10A,10B,10C as well)
k<-0
annual.return$bucket<-NA
for (i in 1:length(funds.per.year)){
  n<-funds.per.year[i]
  bucket.size<-round(n/10, digits=0)
  extreme.size<-round(bucket.size/3, digits=0)
  annual.return$bucket[(k+1):(k+extreme.size)]<-"bC"
  k<-k+extreme.size
  annual.return$bucket[(k+1):(k+extreme.size)]<-"bB"
  k<-k+extreme.size
  annual.return$bucket[(k+1):(k+bucket.size-2*extreme.size)]<-"bA"
  k<-(k+bucket.size-2*extreme.size)
  annual.return$bucket[(k+1):(k+bucket.size)]<-"9"
  k<-(k+bucket.size)
  annual.return$bucket[(k+1):(k+bucket.size)]<-"8"
  k<-(k+bucket.size)
  annual.return$bucket[(k+1):(k+bucket.size)]<-"7"
  k<-(k+bucket.size)
  annual.return$bucket[(k+1):(k+bucket.size)]<-"6"
  k<-(k+bucket.size)
  annual.return$bucket[(k+1):(k+bucket.size)]<-"5"
  k<-(k+bucket.size)
  annual.return$bucket[(k+1):(k+bucket.size)]<-"4"
  k<-(k+bucket.size)
  annual.return$bucket[(k+1):(k+bucket.size)]<-"3"
  k<-(k+bucket.size)
  annual.return$bucket[(k+1):(k+bucket.size)]<-"2"
  k<-(k+bucket.size)
  extreme.size<-round((funds.per.year[i]-9*bucket.size)/3, digits=0)   # find number left over, and divide into 3 buckets
  annual.return$bucket[(k+1):(k+extreme.size)]<-"tC"
  k<-(k+extreme.size)
  annual.return$bucket[(k+1):(k+extreme.size)]<-"tB"
  k<-(k+extreme.size)
  annual.return$bucket[(k+1):(k+funds.per.year[i]-9*bucket.size-2*extreme.size)]<-"tA"
  k<-(k+funds.per.year[i]-9*bucket.size-2*extreme.size)
}
annual.return$bucket<-paste0(annual.return$year, ".",annual.return$bucket)


# create array to merge with original data for calculating monthly return on portfolios in next year
N<-nrow(annual.return)
monthly.returns<-data.frame(fund_id=character(N*12), year=character(N*12), month=character(N*12), bucket=character(N*12), stringsAsFactors = FALSE)
monthly.returns$fund_id<-rep(annual.return$fund_id, each=12)
monthly.returns$year<-rep(annual.return$year+1, each=12)
months<-c(1:12)
monthly.returns$month<-rep(months, times=N)
monthly.returns$bucket<-rep(annual.return$bucket, each=12)
# merge monthly.returns with Data to find the returns on each fund in the next year
Data<-Data[,c("fund_id","year","month","return")]
monthly.returns<-left_join(monthly.returns, Data)
# remove all with NA returns
monthly.returns<-monthly.returns[!is.na(monthly.returns$return),]

# find average monthly returns per bucket
monthly.returns$avgreturn<-with(monthly.returns, ave(monthly.returns$return, monthly.returns$bucket, monthly.returns$month, FUN=mean))
# find just top and bucket bucket (combining 1A,1B,1C for the top bucket, and 10A,10B,10C for the bottom bucket)
monthly.returns.top<-monthly.returns[which(substr(monthly.returns$bucket,6,6)=="t"),]
monthly.returns.top$bucket<-substr(monthly.returns.top$bucket,1,6)
monthly.returns.bot<-monthly.returns[which(substr(monthly.returns$bucket,6,6)=="b"),]
monthly.returns.bot$bucket<-substr(monthly.returns.bot$bucket,1,6)
# calculate average returns in top and bot buckets
monthly.returns.top$avgreturn<-with(monthly.returns.top, ave(monthly.returns.top$return, monthly.returns.top$bucket, monthly.returns.top$month, FUN=mean))
monthly.returns.bot$avgreturn<-with(monthly.returns.bot, ave(monthly.returns.bot$return, monthly.returns.bot$bucket, monthly.returns.bot$month, FUN=mean))
monthly.returns<-rbind(monthly.returns, monthly.returns.top, monthly.returns.bot)
# make average.return data frame, which is the monthly equal-weighted return on each portfolio of funds in the next year 
average.return<-monthly.returns[!duplicated(paste0(monthly.returns$bucket, monthly.returns$month)), colnames(monthly.returns) %in% c("year","month","bucket","avgreturn")]
rm(monthly.returns, monthly.returns.top, monthly.returns.bot, annual.return, Data, keep.values, fylength, fypair)

# import factor returns
factors<-read.table("factors.csv", header=TRUE, sep=",", stringsAsFactors = FALSE)
factors$year<-as.numeric(substr(factors$Date,1,4))
factors$month<-as.numeric(substr(factors$Date,5,6))
# merge factor returns with average.return
return.with.factors<-left_join(average.return, factors)
# change returns in each portfolio into excess returns and scale by 100
return.with.factors$exreturn<-100*(return.with.factors$avgreturn-1)-return.with.factors$RF
# get rid of the year in the bucket column
return.with.factors$bucket<-substr(return.with.factors$bucket,6,nchar(return.with.factors$bucket))
# order return.with.factors by bucket, then year, then month
return.with.factors<-return.with.factors[order(return.with.factors$bucket, return.with.factors$year, return.with.factors$month),]
# find spreads as in Carhart (t-b, tA-bC, 9-b)
rwf.t<-return.with.factors[which(return.with.factors$bucket=="t"),]
rwf.b<-return.with.factors[which(return.with.factors$bucket=="b"),]
rwf.tA<-return.with.factors[which(return.with.factors$bucket=="tA"),]
rwf.bC<-return.with.factors[which(return.with.factors$bucket=="bC"),]
rwf.9<-return.with.factors[which(return.with.factors$bucket=="9"),]
# 1-10 spread
rwf.tb<-rwf.t
rwf.tb$bucket<-paste0(rwf.tb$bucket,"b")
rwf.tb$exreturn<-rwf.t$exreturn-rwf.b$exreturn
# 1A-10C spread
rwf.tAbC<-rwf.tA
rwf.tAbC$bucket<-paste0(rwf.tA$bucket,"bC")
rwf.tAbC$exreturn<-rwf.tA$exreturn-rwf.bC$exreturn
# 9-10 spread
rwf.9b<-rwf.9
rwf.9b$bucket<-paste0(rwf.9$bucket,"b")
rwf.9b$exreturn<-rwf.9$exreturn-rwf.b$exreturn
# append all of them to return.with.factors
return.with.factors<-rbind(return.with.factors, rwf.tb, rwf.tAbC, rwf.9b)
return.with.factors<-return.with.factors[,colnames(return.with.factors) %in% c("year","month","bucket","VWRF","RMRF","SMB","HML","RF","MOM","exreturn")]
rm(rwf.9, rwf.9b, rwf.b, rwf.bC, rwf.t, rwf.tA, rwf.tAbC, rwf.tb, average.return)



# TABLES AND FIGURES
# regressions for 1963-1993 [Table 1, replication]
buckets<-c("tA","tB","tC","t","2","3","4","5","6","7","8","9","b","bA","bB","bC","tb","tAbC","9b")
coef.pre<-matrix(,nrow=19, ncol=17, dimnames=list(NULL, c("Bucket","Monthly Return","StD Return","CAPM Alpha", "CAPM Alpha t-stat", "CAPM Beta", "CAPM Beta t-stat", 
                                                          "4Factor Alpha","4Factor Alpha t-stat","4Factor Beta", "4Factor Beta t-stat","SMB Beta", "SMB t-stat","HML Beta","HML t-stat","Mom Beta", "Mom t-stat")))
for (i in 1:length(buckets)){
  coef.pre[i,1]<-buckets[i]
  # excess return and std
  coef.pre[i,2]<-round(mean(return.with.factors$exreturn[which(return.with.factors$bucket==buckets[i] & return.with.factors$year<=1993)], na.rm=TRUE),2)
  coef.pre[i,3]<-round(sd(return.with.factors$exreturn[which(return.with.factors$bucket==buckets[i] & return.with.factors$year<=1993)], na.rm=TRUE),2)
  # CAPM regression
  reg<-lm(exreturn~VWRF, bucket==buckets[i] & year<=1993, data=return.with.factors)
  coef.pre[i,4]<-round(summary(reg)$coefficients[1,1],2)
  coef.pre[i,5]<-round(summary(reg)$coefficients[1,3],2)
  coef.pre[i,6]<-round(summary(reg)$coefficients[2,1],2)
  coef.pre[i,7]<-round(summary(reg)$coefficients[2,3],2)
  # 4 factor regression
  reg<-lm(exreturn~RMRF+SMB+HML+MOM, bucket==buckets[i] & year<=1993, data=return.with.factors)
  coef.pre[i,8]<-round(summary(reg)$coefficients[1,1],2)
  coef.pre[i,9]<-round(summary(reg)$coefficients[1,3],2)
  coef.pre[i,10]<-round(summary(reg)$coefficients[2,1],2)
  coef.pre[i,11]<-round(summary(reg)$coefficients[2,3],2)
  coef.pre[i,12]<-round(summary(reg)$coefficients[3,1],2)
  coef.pre[i,13]<-round(summary(reg)$coefficients[3,3],2)
  coef.pre[i,14]<-round(summary(reg)$coefficients[4,1],2)
  coef.pre[i,15]<-round(summary(reg)$coefficients[4,3],2)
  coef.pre[i,16]<-round(summary(reg)$coefficients[5,1],2)
  coef.pre[i,17]<-round(summary(reg)$coefficients[5,3],2)
}

# regressions for 1994-2018 [Table 1, out of sample]
coef.post<-matrix(,nrow=19, ncol=17, dimnames=list(NULL, c("Bucket","Monthly Return","StD Return","CAPM Alpha", "CAPM Alpha t-stat", "CAPM Beta", "CAPM Beta t-stat", 
                                                           "4Factor Alpha","4Factor Alpha t-stat","4Factor Beta", "4Factor Beta t-stat","SMB Beta", "SMB t-stat","HML Beta","HML t-stat","Mom Beta", "Mom t-stat")))
for (i in 1:length(buckets)){
  coef.post[i,1]<-buckets[i]
  # excess return and std
  coef.post[i,2]<-round(mean(return.with.factors$exreturn[which(return.with.factors$bucket==buckets[i] & return.with.factors$year>1993)], na.rm=TRUE),2)
  coef.post[i,3]<-round(sd(return.with.factors$exreturn[which(return.with.factors$bucket==buckets[i] & return.with.factors$year>1993)], na.rm=TRUE),2)
  # CAPM regression
  reg<-lm(exreturn~VWRF, bucket==buckets[i] & year>1993, data=return.with.factors)
  coef.post[i,4]<-round(summary(reg)$coefficients[1,1],2)
  coef.post[i,5]<-round(summary(reg)$coefficients[1,3],2)
  coef.post[i,6]<-round(summary(reg)$coefficients[2,1],2)
  coef.post[i,7]<-round(summary(reg)$coefficients[2,3],2)
  # 4 factor regression
  reg<-lm(exreturn~RMRF+SMB+HML+MOM, bucket==buckets[i] & year>1993, data=return.with.factors)
  coef.post[i,8]<-round(summary(reg)$coefficients[1,1],2)
  coef.post[i,9]<-round(summary(reg)$coefficients[1,3],2)
  coef.post[i,10]<-round(summary(reg)$coefficients[2,1],2)
  coef.post[i,11]<-round(summary(reg)$coefficients[2,3],2)
  coef.post[i,12]<-round(summary(reg)$coefficients[3,1],2)
  coef.post[i,13]<-round(summary(reg)$coefficients[3,3],2)
  coef.post[i,14]<-round(summary(reg)$coefficients[4,1],2)
  coef.post[i,15]<-round(summary(reg)$coefficients[4,3],2)
  coef.post[i,16]<-round(summary(reg)$coefficients[5,1],2)
  coef.post[i,17]<-round(summary(reg)$coefficients[5,3],2)
}


# 10-year rolling 1-factor alphas for the fund portfolios [Figure 1 and Figure 2]
years<-c(1972:2018)
rollinga<-data.frame(year=numeric(length(years)), top_alpha=numeric(length(years)), top_tstat=numeric(length(years)), bot_alpha=numeric(length(years)), bot_tstat=numeric(length(years)), tb_alpha=numeric(length(years)), tb_tstat=numeric(length(years)), stringsAsFactors = FALSE)
for (i in 1:length(years)){
  rollinga$year[i]<-years[i]
  reg<-lm(exreturn~VWRF, bucket=="t" & year>=(years[i]-9) & year<=years[i], data=return.with.factors)
  rollinga$top_alpha[i]<-round(summary(reg)$coefficients[1,1],3)
  rollinga$top_tstat[i]<-round(summary(reg)$coefficients[1,3],3)
  reg<-lm(exreturn~VWRF, bucket=="b" & year>=(years[i]-9) & year<=years[i], data=return.with.factors)
  rollinga$bot_alpha[i]<-round(summary(reg)$coefficients[1,1],3)
  rollinga$bot_tstat[i]<-round(summary(reg)$coefficients[1,3],3)
  reg<-lm(exreturn~VWRF, bucket=="tb" & year>=(years[i]-9) & year<=years[i], data=return.with.factors)
  rollinga$tb_alpha[i]<-round(summary(reg)$coefficients[1,1],3)
  rollinga$tb_tstat[i]<-round(summary(reg)$coefficients[1,3],3)
}

# 10-year rolling 4-factor alphas for the fund portfolios [Figure 5 and Figure 6]
rollinga4<-data.frame(year=numeric(length(years)), top_alpha=numeric(length(years)), top_tstat=numeric(length(years)), bot_alpha=numeric(length(years)), bot_tstat=numeric(length(years)), tb_alpha=numeric(length(years)), tb_tstat=numeric(length(years)), stringsAsFactors = FALSE)
for (i in 1:length(years)){
  rollinga4$year[i]<-years[i]
  reg<-lm(exreturn~RMRF+SMB+HML+MOM, bucket=="t" & year>=(years[i]-9) & year<=years[i], data=return.with.factors)
  rollinga4$top_alpha[i]<-round(summary(reg)$coefficients[1,1],3)
  rollinga4$top_tstat[i]<-round(summary(reg)$coefficients[1,3],3)
  reg<-lm(exreturn~RMRF+SMB+HML+MOM, bucket=="b" & year>=(years[i]-9) & year<=years[i], data=return.with.factors)
  rollinga4$bot_alpha[i]<-round(summary(reg)$coefficients[1,1],3)
  rollinga4$bot_tstat[i]<-round(summary(reg)$coefficients[1,3],3)
  reg<-lm(exreturn~RMRF+SMB+HML+MOM, bucket=="tb" & year>=(years[i]-9) & year<=years[i], data=return.with.factors)
  rollinga4$tb_alpha[i]<-round(summary(reg)$coefficients[1,1],3)
  rollinga4$tb_tstat[i]<-round(summary(reg)$coefficients[1,3],3)
}

# 10-year rolling loadings on HML and MOM of t-b fund portfolio in 4-factor regression [Figure 3]
rollingloadings<-data.frame(year=numeric(length(years)), hml=numeric(length(years)), mom=numeric(length(years)))
for (i in 1:length(years)){
  rollingloadings$year[i]<-years[i]
  reg<-lm(exreturn~RMRF+SMB+HML+MOM, bucket=="tb" & year>=(years[i]-9) & year<=years[i], data=return.with.factors)
  rollingloadings$hml[i]<-round(summary(reg)$coefficients[4,1],3)
  rollingloadings$mom[i]<-round(summary(reg)$coefficients[5,1],3)
}


# 10-year rolling one-factor alphas of HML and MOM factor portfolios [Figure 4]
rollinga1<-data.frame(year=numeric(length(years)), hml_alpha=numeric(length(years)), hml_tstat=numeric(length(years)), mom_alpha=numeric(length(years)), mom_tstat=numeric(length(years)), stringsAsFactors = FALSE)
for (i in 1:length(years)){
  rollinga1$year[i]<-years[i]
  reg<-lm(HML~VWRF, year>=(years[i]-9) & year<=years[i], data=factors)
  rollinga1$hml_alpha[i]<-round(summary(reg)$coefficients[1,1],3)
  rollinga1$hml_tstat[i]<-round(summary(reg)$coefficients[1,3],3)
  reg<-lm(MOM~VWRF, year>=(years[i]-9) & year<=years[i], data=factors)
  rollinga1$mom_alpha[i]<-round(summary(reg)$coefficients[1,1],3)
  rollinga1$mom_tstat[i]<-round(summary(reg)$coefficients[1,3],3)
}


# [Untablulated Results]
# regressions for full sample (1963-2018)
coef.full<-matrix(,nrow=19, ncol=17, dimnames=list(NULL, c("Bucket","Monthly Return","StD Return","CAPM Alpha", "CAPM Alpha t-stat", "CAPM Beta", "CAPM Beta t-stat", 
                                                           "4Factor Alpha","4Factor Alpha t-stat","4Factor Beta", "4Factor Beta t-stat","SMB Beta", "SMB t-stat","HML Beta","HML t-stat","Mom Beta", "Mom t-stat")))
for (i in 1:length(buckets)){
  coef.full[i,1]<-buckets[i]
  # excess return and std
  coef.full[i,2]<-round(mean(return.with.factors$exreturn[which(return.with.factors$bucket==buckets[i])], na.rm=TRUE),2)
  coef.full[i,3]<-round(sd(return.with.factors$exreturn[which(return.with.factors$bucket==buckets[i])], na.rm=TRUE),2)
  # CAPM regression
  reg<-lm(exreturn~VWRF, bucket==buckets[i], data=return.with.factors)
  coef.full[i,4]<-round(summary(reg)$coefficients[1,1],2)
  coef.full[i,5]<-round(summary(reg)$coefficients[1,3],2)
  coef.full[i,6]<-round(summary(reg)$coefficients[2,1],2)
  coef.full[i,7]<-round(summary(reg)$coefficients[2,3],2)
  # 4 factor regression
  reg<-lm(exreturn~RMRF+SMB+HML+MOM, bucket==buckets[i], data=return.with.factors)
  coef.full[i,8]<-round(summary(reg)$coefficients[1,1],2)
  coef.full[i,9]<-round(summary(reg)$coefficients[1,3],2)
  coef.full[i,10]<-round(summary(reg)$coefficients[2,1],2)
  coef.full[i,11]<-round(summary(reg)$coefficients[2,3],2)
  coef.full[i,12]<-round(summary(reg)$coefficients[3,1],2)
  coef.full[i,13]<-round(summary(reg)$coefficients[3,3],2)
  coef.full[i,14]<-round(summary(reg)$coefficients[4,1],2)
  coef.full[i,15]<-round(summary(reg)$coefficients[4,3],2)
  coef.full[i,16]<-round(summary(reg)$coefficients[5,1],2)
  coef.full[i,17]<-round(summary(reg)$coefficients[5,3],2)
}


# find 1-factor alpha at each cutoff year (for year tau to 2018)
# significant for 1963 <= tau <= 1980
# insignifcant for tau > 1982
years<-c(1963:2018)
alphas<-data.frame(year=numeric(length(years)), top_alpha=numeric(length(years)), top_tstat=numeric(length(years)), tb_alpha=numeric(length(years)), tb_tstat=numeric(length(years)), stringsAsFactors = FALSE)
for (i in 1:length(years)){
  alphas$year[i]<-years[i]
  # top bin
  reg<-lm(exreturn~VWRF, bucket=="t" & year>=years[i], data=return.with.factors)
  alphas$top_alpha[i]<-round(summary(reg)$coefficients[1,1],3)
  alphas$top_tstat[i]<-round(summary(reg)$coefficients[1,3],3)
  # top-bot bin
  reg<-lm(exreturn~VWRF, bucket=="tb" & year>=years[i], data=return.with.factors)
  alphas$tb_alpha[i]<-round(summary(reg)$coefficients[1,1],3)
  alphas$tb_tstat[i]<-round(summary(reg)$coefficients[1,3],3)
}


# difference in loadings on HML and MOM factors in pre and post period
# hml and mom are loading in the post-period
# diffhml and diffmom are the difference between loading in the pre-period and loading in the post-period
return.with.factors$pre<-0
return.with.factors$pre[which(return.with.factors$year<=1993)]<-1
return.with.factors$preRMRF<-return.with.factors$pre*return.with.factors$RMRF
return.with.factors$preSMB<-return.with.factors$pre*return.with.factors$SMB
return.with.factors$preHML<-return.with.factors$pre*return.with.factors$HML
return.with.factors$preMOM<-return.with.factors$pre*return.with.factors$MOM
diffloading<-data.frame(bucket=character(19), hml=numeric(19), hml_tstat=numeric(19), diffhml=numeric(19), diffhml_tstat=numeric(19), mom=numeric(19), mom_tstat=numeric(19), diffmom=numeric(19), diffmom_tstat=numeric(19), stringsAsFactors = FALSE)
for (i in 1:length(buckets)){
  diffloading$bucket[i]<-buckets[i]
  reg<-lm(exreturn~RMRF+SMB+HML+MOM+preRMRF+preSMB+preHML+preMOM+pre, bucket=="tb", data=return.with.factors)
  diffloading$hml[i]<-round(summary(reg)$coefficients[4,1],2)
  diffloading$hml_tstat[i]<-round(summary(reg)$coefficients[4,3],2)
  diffloading$diffhml[i]<-round(summary(reg)$coefficients[8,1],2)
  diffloading$diffhml_tstat[i]<-round(summary(reg)$coefficients[8,3],2)
  diffloading$mom[i]<-round(summary(reg)$coefficients[5,1],2)
  diffloading$mom_tstat[i]<-round(summary(reg)$coefficients[5,3],2)
  diffloading$diffmom[i]<-round(summary(reg)$coefficients[9,1],2)
  diffloading$diffmom_tstat[i]<-round(summary(reg)$coefficients[9,3],2)
}